@@ -27,12 +27,12 @@ static void      gail_image_class_init         (GailImageClass *klass);
 static void      gail_image_init               (GailImage      *image);
 static void      gail_image_initialize         (AtkObject       *accessible,
                                                 gpointer        data);
-static G_CONST_RETURN gchar* gail_image_get_name  (AtkObject     *accessible);
+static const gchar* gail_image_get_name  (AtkObject     *accessible);
 
 
 static void      atk_image_interface_init      (AtkImageIface  *iface);
 
-static G_CONST_RETURN gchar *
+static const gchar *
                  gail_image_get_image_description (AtkImage     *image);
 static void	 gail_image_get_image_position    (AtkImage     *image,
                                                    gint         *x,
@@ -118,7 +118,7 @@ elide_underscores (const gchar *original)
   return result;
 }
 
-static G_CONST_RETURN gchar*
+static const gchar*
 gail_image_get_name (AtkObject *accessible)
 {
   GtkWidget* widget;
@@ -146,9 +146,11 @@ gail_image_get_name (AtkObject *accessible)
   g_free (image_accessible->stock_name);
   image_accessible->stock_name = NULL;
 
+  if (gtk_image_get_storage_type (image) != GTK_IMAGE_STOCK)
+    return NULL;
+
   gtk_image_get_stock (image, &stock_id, NULL);
-  if (gtk_image_get_storage_type (image) != GTK_IMAGE_STOCK ||
-      stock_id == NULL)
+  if (stock_id == NULL)
     return NULL;
 
   if (!gtk_stock_lookup (stock_id, &stock_item))
@@ -167,7 +169,7 @@ atk_image_interface_init (AtkImageIface *iface)
   iface->set_image_description = gail_image_set_image_description;
 }
 
-static G_CONST_RETURN gchar * 
+static const gchar *
 gail_image_get_image_description (AtkImage     *image)
 {
   GailImage* aimage = GAIL_IMAGE (image);
@@ -207,13 +209,6 @@ gail_image_get_image_size (AtkImage *image,
   image_type = gtk_image_get_storage_type(gtk_image);
  
   switch(image_type) {
-    case GTK_IMAGE_PIXMAP:
-    {	
-      GdkPixmap *pixmap;
-      gtk_image_get_pixmap(gtk_image, &pixmap, NULL);
-      gdk_drawable_get_size (pixmap, width, height);
-      break;
-    }
     case GTK_IMAGE_PIXBUF:
     {
       GdkPixbuf *pixbuf;
