@@ -133,7 +133,7 @@ static void       gail_text_view_paste_received        (GtkClipboard     *clipbo
 /* AtkStreamableContent */
 static void       atk_streamable_content_interface_init    (AtkStreamableContentIface *iface);
 static gint       gail_streamable_content_get_n_mime_types (AtkStreamableContent *streamable);
-static G_CONST_RETURN gchar* gail_streamable_content_get_mime_type (AtkStreamableContent *streamable,
+static const gchar* gail_streamable_content_get_mime_type (AtkStreamableContent *streamable,
 								    gint i);
 static GIOChannel* gail_streamable_content_get_stream       (AtkStreamableContent *streamable,
 							     const gchar *mime_type);
@@ -508,7 +508,6 @@ gail_text_view_get_offset_at_point (AtkText      *text,
                                     AtkCoordType coords)
 {
   GtkTextView *view;
-  GtkTextBuffer *buffer;
   GtkTextIter loc_itr;
   gint x_widget, y_widget, x_window, y_window, buff_x, buff_y;
   GtkWidget *widget;
@@ -521,7 +520,6 @@ gail_text_view_get_offset_at_point (AtkText      *text,
     return -1;
 
   view = GTK_TEXT_VIEW (widget);
-  buffer = gtk_text_view_get_buffer (view);
 
   window = gtk_text_view_get_window (view, GTK_TEXT_WINDOW_WIDGET);
   gdk_window_get_origin (window, &x_widget, &y_widget);
@@ -1664,7 +1662,7 @@ static gint       gail_streamable_content_get_n_mime_types (AtkStreamableContent
     return n_mime_types;
 }
 
-static G_CONST_RETURN gchar*       
+static const gchar*
 gail_streamable_content_get_mime_type (AtkStreamableContent *streamable, gint i)
 {
     if (GAIL_IS_TEXT_VIEW (streamable) && GAIL_TEXT_VIEW (streamable)->textutil)
